// Copyright 2004-2005 Castle Project - http://www.castleproject.org/
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
// 
//     http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

namespace Castle.DynamicProxy.Builder.CodeBuilder.SimpleAST
{
	using System;
	using System.Reflection.Emit;

	using Castle.DynamicProxy.Builder.CodeBuilder.Utils;

	/// <summary>
	/// Summary description for FixedReference.
	/// </summary>
	public class FixedReference : TypeReference
	{
		private object _value;

		public FixedReference( object value ) : base( value.GetType() )
		{
			if (!value.GetType().IsPrimitive && !(value is String))
			{
				throw new ApplicationException("Invalid type to FixedReference");
			}

			_value = value;
		}

		public override void Generate(ILGenerator gen)
		{
		}

		public override void LoadReference(ILGenerator gen)
		{
			OpCodeUtil.ConvertValueToLdcOpCode(gen, _value);
		}

		public override void StoreReference(ILGenerator gen)
		{
			throw new NotImplementedException("FixedReference.StoreReference");
		}
	}
}
