using System;
using System.Reflection;
using System.IO;
using Chat.Controlador;

namespace Chat.Remoting
{
	/// <summary>
	/// Descripcin breve de ChatFactory.
	/// </summary>
	public class RemotingChatFactory : IChatFactory
	{
		String fitxerConfig;
		IChatListeners listeners;

		public RemotingChatFactory(string fitxerConfig, IChatListeners listeners)
		{
			this.fitxerConfig = fitxerConfig;
			this.listeners = listeners;
		}

		public RemotingChatFactory(IChatListeners listeners)
		{
			this.listeners = listeners;
			this.fitxerConfig = null;
		}

		[ChatConfig("../../RemotingClient.config")]
		public IChatClient crearChatClient()
		{
			// Obtenim atribut
			MethodInfo metode = typeof (RemotingChatFactory).GetMethod("crearChatClient");
			object [] attributes = metode.GetCustomAttributes (typeof (ChatConfigAttribute), false);
			//if (attributes != null && attributes.Length > 0)
			//{
			ChatConfigAttribute chatConfig = (ChatConfigAttribute)attributes[0];
			//}

			// Si no s'ha especificat un fitxer de configuraci, agafem el fitxer per defecte de l'atribut
			if ((fitxerConfig == null) || (fitxerConfig.CompareTo("") == 0))
			{
				this.fitxerConfig = chatConfig.Fitxer;
			} 
			else 
			{
				chatConfig.Fitxer = this.fitxerConfig;
			}

			return new RemotingChatClient(fitxerConfig, listeners);
		}

		[ChatConfig("../../RemotingServer.config")]
		public IChatServer crearChatServer()
		{
			// Obtenim atribut
			MethodInfo metode = typeof (RemotingChatFactory).GetMethod("crearChatServer");
			object [] attributes = metode.GetCustomAttributes (typeof (ChatConfigAttribute), false);
			//if (attributes != null && attributes.Length > 0)
			//{
			ChatConfigAttribute chatConfig = (ChatConfigAttribute)attributes[0];
			//}

			// Si no s'ha especificat un fitxer de configuraci, agafem el fitxer per defecte de l'atribut
			if ((fitxerConfig == null) || (fitxerConfig.CompareTo("") == 0))
			{
				this.fitxerConfig = chatConfig.Fitxer;
			} 
			else 
			{
				chatConfig.Fitxer = this.fitxerConfig;
			}

			return new RemotingChatServer(fitxerConfig);
		}

	}
}
