using System;
using System.Collections;
using System.Windows.Forms;
using Castle.DynamicProxy;
using Chat.Controlador;

namespace Chat.VistaClient
{
	/// <summary>
	/// Descripcin breve de ChatInterceptor.
	/// </summary>
	public class ChatInterceptor : IInterceptor
	{
		private FormVistaClient window;

		public ChatInterceptor(FormVistaClient window)
		{
			this.window = window;
		}

		public object Intercept(IInvocation invocation, params object[] args)
		{
			if (invocation.Method.Name.Equals("sendCommand"))
			{
				if ((args != null) && (args.Length > 0)) 
				{
					ChatClientCommand command = args[0] as ChatClientCommand;
					if (command != null) 
					{
						
						switch(command.Accio) 
						{
							case Chat.Controlador.TipusAccio.CrearCanal:
								window.escriureLog("CrearCanal: " + command.Canal);
								break;
							case Chat.Controlador.TipusAccio.Entrar:
								window.escriureLog("Entrar: [canal]" + command.Canal + " [usuari]" + command.Nick);
								break;
							case Chat.Controlador.TipusAccio.EsborrarCanal:
								window.escriureLog("EsborrarCanal: " + command.Canal);
								break;
							case Chat.Controlador.TipusAccio.NouMissatge:
								window.escriureLog("NouMissatge: [canal]" + command.Canal + " [usuari]" + command.Nick + " [missatge]" + command.Missatge);
								break;
							case Chat.Controlador.TipusAccio.ObtenirCanals:
								window.escriureLog("ObtenirCanals:");
								break;
							case Chat.Controlador.TipusAccio.ObtenirUsuarisCanal:
								window.escriureLog("ObtenirUsuarisCanal: " + command.Canal);
								break;
							case Chat.Controlador.TipusAccio.Sortir:
								window.escriureLog("Sortir: [canal]" + command.Canal + " [usuari]" + command.Nick);
								break;
						}
					}
				}
			}

			// Crida al objecte original
			object retValue = invocation.Proceed( args );

			if (retValue != null) 
			{
				ArrayList resultat = retValue as ArrayList;
				if (resultat != null) 
				{
					string log = "Retornat: ";
					foreach(string cadena in resultat){
						log = log + cadena + " ";
					}
					window.escriureLog(log);
				}
			}
			return retValue;
		}

	}
}
